package Atm_Moran;
use Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(Fm_Dry_Moran Fm_Wet_Moran Pv_sat Excess_Moran_path Excess_Moran_del Excess_Chao_path Excess_Chao_del);

BEGIN{
    $PI = atan2(1,1)*4.;
    $C =  299792458.;
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}



sub Fm_CFA($$$$$){
    my ($this, $el, $t, $p, $h) = @_;
    
    my $beta = -5.6;
    my $dbeta = $beta + 6.5;
    my $h_tr = 13.; # $BBPN.7w$N9b$5(B (km)
    my $d_h_tr = $h_tr - 11.231;
    my $dtemp = $t - 20.;
    my $dpres = $p - 1000;
    
    my $c = -0.009;
    my $b = 0.001144 * (1. + 0.00001164*$dpres 
			+ 0.0002795 * $pv0 
			+ 0.003109*$dtemp + 0.03038 * $dbeta
			- 0.01217* $d_h_tr);
    my $a = 0.001185 * (1. + 0.00006071 * $dpres - 0.0001471 * $pv0
			+ 0.003072 * $dtemp + 0.01965 * $dbeta
			- 0.005645 * $d_h_tr);
    my $el_rad = $el /180/ * $PI;
    
    my $fm = 1./(sin($el_rad) + $a/(tan($el_rad) + $b/(sin($el_rad) + $c)));

}

# Mapping function of Dry Atm by Chao
sub Fm_Dry_Chao($$){
    ($this, $el) = @_;
    my $el_rad = $el/180*$PI;
    
    my $tane = sin($el_rad)/cos($el_rad);
    my $fm1 = 0.9922498/(sin($el_rad)+(0.00143/($tane + 0.0445)));

    return $fm1;
}

# Mapping function of Wet Atm by Chao
sub Fm_Wet_Chao($$){
    ($this, $el) = @_;
    my $el_rad = $el/180*$PI;
    
    my $tane = sin($el_rad)/cos($el_rad);
    my $fm1 = 1./(sin($el_rad)+(0.00035/($tane + 0.017)));

    return $fm1;
}

# Mapping function of Dry Atm by Moran
sub Fm_Dry_Moran($$){
    ($this, $el) = @_;
    my $el_rad = $el/180*$PI;
    my $z = $PI*0.5 - $el_rad;
    
    my $tanz = sin($z)/cos($z);
    my $fm1 = (1. - 0.0013*$tanz*$tanz)/sin($z);
    return $fm1;
}

# Mapping function of Wet Atm by Moran
sub Fm_Wet_Moran($$){
    ($this, $el) = @_;

    my $el_rad = $el/180*$PI;
    my $z = $PI*0.5 - $el_rad;
        my $tanz = sin($z)/cos($z);
    my $fm2 = (1. - 0.0003 * $tanz * $tanz )/sin($z);

}


# Compute Saturation parital Pressure of Water Vapor 
sub Pv_sat($$){ 
    my ($this,$t) = @_;
    my $a = 7.5 * $t / ($t + 237.3);
    my $b = 6.11 * 10^$a;
}

# Compute Zenith Excess path length in (m)
sub Excess_Moran_path($$$){
    my ($this,$t, $p, $h) = @_;
    
    my $pvs = &Pv_sat($t);
    my $pv0 = $pvs * $h * 0.01;
    my $del_dry = 0.228 * $p;
    my $tabs = $t + 273.15;
    my $del_wet = 7.5e4 * $pv0/$tabs/$tabs;

    return ($del_dry*0.01, $del_wet*0.01);
}

# Compute Zenith Excess path length in (m)
sub Excess_Chao_path($$$){
    my ($this,$t, $p, $h) = @_;
    
    my $pvs = &Pv_sat($t);
    my $pv0 = $pvs * $h * 0.01;
    my $del_dry = 2.2768e-3 * $p;
    my $tabs = $t + 273.15;
    my $del_wet = 2.2777*(1255./$tabs + 0.05) * $pv0;

    return ($del_dry, $del_wet);
}

# Compute Zenith Excess path delay in (sec)
sub Excess_Chao_del($$$$){
    my ($this,$t, $p, $h) = @_;
    
    my $pvs = $this->Pv_sat($t);
    my $pv0 = $pvs * $h * 0.01;
    my $del_dry = 2.2768e-3 * $p;
    my $tabs = $t + 273.15;
    my $del_wet = 2.2777e-3*(1255./$tabs + 0.05) * $pv0;
    $del_dry /= $C;
    $del_wet /= $C;
    return ($del_dry, $del_wet);
}

sub Excess_Moran_del($$$){
    my ($this,$t, $p, $h) = @_;
    
    my $pvs = &Pv_sat($t);
    my $pv0 = $pvs * $h * 0.01;
    my $del_dry = 0.228 * $p;
    my $tabs = $t + 273.15;
    my $del_wet = 7.5e4 * $pv0/$tabs/$tabs;
    
    $del_dry *= 0.01/$C;
    $del_wet *= 0.01/$C;
    return ($del_dry, $del_wet);
}
