package Cphe_fmt;
 # Corrected delay data residual format.
 # yy/dd hms src frq amp del-rsd delerr, rate, raterr, snr, orig-rsd, elx, ely (deg)
 # 2005/308 02:08:43.000  HYBS0209 8408300410.80 2.918e-05  0.000000000000e+00 2.08e+01 0.0 0.0    1.20 -2.199845681161e-06  41.74  50.35

use Exporter;
use Atm_Moran;
use Time_lib;
@ISA = qw(Exporter);

@EXPORT = qw();
# @EXPORT = qw(Read);

BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Tdat($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $this->{ndat} || $i < 0){
	print STDERR "Error: Tdat($i) is out of range (< $this->{ndat})\n";
	exit(1);
    }
    
    return ${$this->{tarr}}[$i];
}

sub Frq($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};

    if($i >= $ndat || $i < 0){
	print STDERR "Error: Frq($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{frq}}[$i];
}
sub Rate($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $ndat || $i < 0){
	print STDERR "Error: Frq($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{rate}}[$i];
}

sub Raterr($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $ndat || $i < 0){
	print STDERR "Error: Frq($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{rerr}}[$i];
}

sub Ndat($){
    my $this = shift;
    
    return $this->{ndat};
}

sub Srcname($$){
    my ($this, $i) = @_;
    my $ndat =  $this->{ndat};

    if($i >= $ndat || $i < 0){
	print STDERR "Error: Srcname($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{src}}[$i];
}

sub Amp($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};

    if($i >= $ndat || $i < 0){
	print STDERR "Error: Amp($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{amp}}[$i];
}

sub Snr($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};

    if($i >= $ndat || $i < 0){
	print STDERR "Error: Snr($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{snr}}[$i];
}

sub Elx($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $this->{ndat} || $i < 0){
	print STDERR "Error: Elx($i) is out of range (<$ndat)\n";
	exit(1);
    }
    unless(defined(${$this->{elx}}[$i])){
	print STDERR "Error: Elx[$i] is not defined \n";
	exit(1);
    }
    return ${$this->{elx}}[$i];
}
sub Ely($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $this->{ndat} || $i < 0){
	print STDERR "Error: Ely($i) is out of range (<$ndat)\n";
	exit(1);
    }
    unless(defined(${$this->{elx}}[$i])){
	print STDERR "Error: Ely[$i] is not defined \n";
	exit(1);
    }
    return ${$this->{elx}}[$i];
}

sub Del($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $this->{ndat} || $i < 0){
	print STDERR "Error: corrected_del[$i] is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{del}}[$i];
}

sub Delerr($$){
    my ($this, $i) = @_;
    my $ndat = $this->{ndat};
    if($i >= $ndat || $i < 0){
	print STDERR "Error: delerr($i) is out of range (<$ndat)\n";
	exit(1);
    }
    return ${$this->{err}}[$i];
}

sub Read($$){
    my ($this, $file) = @_;
    open(IN, $file) || die "Cannot open delay $file \n";
    
    my (@tmarr, @srcname, @frq, @amp,@corrected_del, @rate, @rate_err, @snr, @original_del,
	@elx, @ely);
    my $cnt=0;
    while(<IN>){
	next if(/^\#/);
	my @terms = split(/\s+/);
	next if(@terms < 10);
	my @yd = split(m!/!,$terms[0]);
	my @hms = split(m!:!,$terms[1]);
	$tmarr[$cnt] = new Time_lib;
	$tmarr[$cnt]->Set_time_ydhms($yd[0], $yd[1], $hms[0], $hms[1], $hms[2]);
	$srcname[$cnt] = $terms[2];
	$frq[$cnt] = $terms[3];
	$amp[$cnt] = $terms[4];
	$corrected_del[$cnt] = $terms[5];
	$delerr[$cnt] = $terms[6];
	$rate[$cnt] = $terms[7];
	$rate_err[$cnt] = $terms[8];
	$snr[$cnt] = $terms[9];
	$original_del[$cnt] = $terms[10];
	if(defined($terms[11]) && defined($terms[12])){
	    $elx[$cnt] = $terms[11];
	    $ely[$cnt] = $terms[12];
	}
	$cnt++;
    }
    my $ndat = $cnt;
    close(IN);

    $this->{tarr} = \@tmarr;
    $this->{src} = \@srcname;
    $this->{frq} = \@frq;
    $this->{amp} = \@amp;
    $this->{del} = \@corrected_del;
    $this->{err} = \@delerr;
    $this->{rate} = \@rate;
    $this->{rerr} = \@rate_err;
    $this->{snr} = \@snr;
    $this->{original_del} = \@original_del;
    if( defined($elx[0])){
	$this->{elx} = \@elx;
	$this->{ely} = \@ely;
    }
    $this->{ndat} = $ndat;

}

1;
