package SlvSpool;
## Package for reading Spool file of SOLVE.

BEGIN{

}

use Time_lib;

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Get_stn($$){
    my $this = shift;
    my $in = shift;
    seek( $in, 0, 0);
    my $epoch = new Time_lib;
    my $mode =0;
    my (@xyz, @xyz_err, @xyz_adj); 
    while(<$in>){
	if(/Station positions are for epoch:\s*(\d{4}).(\d+).(\d+)-(\d+):(\d+):(\d+)/){
	    my @ymdhms = ($1,$2,$3,$4,$5,$6);
	    $epoch->Set_time_ymd(@ymdhms);
	    $mode++;
	    next;
	}
	
	if($mode >0){
	    if(/ KASHIM11 7334 NOAM  X Comp\s+/){
		my @term = split(/\s+/,$');   # ');
		$xyz[0] = $term[0];
		$xyz_adj[0] = $term[2];
		$xyz_err[0] = $term[4];
		$mode++;
		next;
	    }
	    if(/ KASHIM11 7334 NOAM  Y Comp\s+/){
		my @term = split(/\s+/,$');   # ');
		$xyz[1] = $term[0];
		$xyz_adj[1] = $term[2];
		$xyz_err[1] = $term[4];
		$mode++;
		next;
	    }
	    if(/ KASHIM11 7334 NOAM  Z Comp\s+/){
		my @term = split(/\s+/,$');   # ');
		$xyz[2] = $term[0];
		$xyz_adj[2] = $term[2];
		$xyz_err[2] = $term[4];
		$mode++;
		last;
	    }
	}
    }

    $this->{epoch} = $epoch;
    $this->{xyz} = \@xyz;
    $this->{xyz_adj} = \@xyz_adj;
    $this->{xyz_err} = \@xyz_err;
    if($mode < 4){
	print STDERR "E XYZ data was not completely obtained.\n";
	return -1;
    }else{
	return 0;
    }
    
}

sub Get_base($$){
    my $this = shift;
    my $in = shift;
    seek( $in, 0, 0);

    my $mode =0;
    my (@xyz, @xyz_err); 
    my $line;
    my @baseline;
    while($line = <$in>){
	if($line =~/Baseline vector components/){
	    $mode=1;	    next;	}
	if($mode >0){
	    if($line =~/KASHIM11 7334       to KOGANEI  7327\s+/){
		my @term = split(/\s+/,$');  # ');
		$baseline[0] = $term[0];
		$baseline[1] = $term[1];
		$this->{baseline} = \@baseline;
		last;
	    }
	}
    }
    
}

sub Read_Resid($$){
    my $this = shift;
    my $in = shift;
    seek( $in, 0, 0);
    
    my $mode =0;
    my (@tarr, @rsdarr, @srcarr, @incflag, @scan );
    my $line;
    my $cnt =0;
    while($line = <$in>){
	if($line =~/Residuals from/){
	    $mode=1;$line = <$in>;  $line = <$in>; 
	    next;	}
	if($mode >0){
	    if($line =~/(\d)\s+([!\@])\s+/){
		my $iscan = $1;
		my $flag = $2;
		my @tms = split(/\s+/,$'); # ');
		if(@tms < 9){
		    print STDERR "Err: Unexpected data: $line.\n";
		    next;
		}
		my @hms = split(/[.,\-,:]/,$tms[8]);
		my $tm = new Time_lib;
		$tm->Set_time_ymd(@hms);
		$tarr[$cnt] = $tm;
		$rsdarr[$cnt] = $tms[4] * 1.e-9;
		$srcarr[$cnt] = $tms[2];
		$scan[$cnt] = $iscan;
		if($flag eq '!' ){ $incflag[$cnt] = 1;}
		else { $incflag[$cnt] = 0;}
		$cnt++;
	    }else{
		last;
	    }
	}
    }

    $this->{rsdarr} = \@rsdarr;
    $this->{srcarr} = \@srcarr;
    $this->{rsdtarr} = \@tarr;
    $this->{flag} = \@incflag;
    $this->{iscan} = \@scan;
}

sub Read_Clk($$){
    my $this = shift;
    my $in = shift;
    seek( $in, 0, 0);
    
    my (@tarr, @clkarr,@clkerr, $cl1, $cl2, $cl1err, $cl2err, $tcl1, $tcl2 );
    my $line;
    my $cnt =0;
    my $stname;
    while($line = <$in>){
	if($line =~/(\S+)\s+cl 0\s+/ || $line =~/(\S+)\s+CL 0\s+/){
	    $stname = $1;
	    my @tms = split(/\s+/,$'); # ');
	    if(@tms < 8){
		print STDERR "Err: cl0 Unexpected data: $line.\n";
		next;
	    }
	    my @ymd = split(m!/!, $tms[0]);
	    my @hm = split(/:/, $tms[1]);
	    my $tm = new Time_lib;
	    $tm->Set_time_ymd( @ymd, @hm, 0);
	    $tarr[$cnt] = $tm;
	    $clkarr[$cnt] = $tms[2] * 1.e-9;
	    $clkerr[$cnt] = $tms[6] * 1.e-9;
	    $cnt++;
	}elsif($line =~/(\S+)\s+cl 1\s+/ || $line =~/(\S+)\s+CL 1\s+/){
	    my @tms = split(/\s+/,$'); # ');
	    if(@tms < 8){
		print STDERR "Err: cl1 Unexpected data: $line.\n";
		next;
	    }
	    my @ymd = split(m!/!, $tms[0]);
	    my @hm = split(/:/, $tms[1]);
	     $tcl1 = new Time_lib;
	    $tcl1->Set_time_ymd( @ymd, @hm, 0);
	    $tms[3] =~/D([-,\d]+)/;
	    my $ord = $1;
	    $cl1 = $tms[2] * 10**($ord);
	    $cl1err = $tms[6] * 10**($ord);
	}elsif($line =~/(\S+)\s+cl 2\s+/ || $line =~/(\S+)\s+CL 2\s+/){
	    my @tms = split(/\s+/,$'); # ');
	    if(@tms < 8){
		print STDERR "Err: cl2 Unexpected data: $line.\n";
		next;
	    }
	    my @ymd = split(m!/!, $tms[0]);
	    my @hm = split(/:/, $tms[1]);
	     $tcl2 = new Time_lib;
	    $tcl2->Set_time_ymd( @ymd, @hm, 0);
	    $tms[3] =~/D([-,\d]+)/;
	    my $ord = $1;
	    $cl2 = $tms[2] * 10**($ord); ## (sec/day)
	    $cl2err = $tms[6] * 10**($ord);
	}
    }

    $this->{clkerr} = \@clkerr;
    $this->{clkarr} = \@clkarr;
    $this->{clkepoch} = \@tarr;
    $this->{cl1} = $cl1;
    $this->{cl2} = $cl2; ## (sec/day)
    $this->{cl1epoch} = $tcl1;
    $this->{cl2epoch} = $tcl2;
    $this->{cl1err} = $cl1err;
    $this->{cl2err} = $cl2err;

}

## mode=0 : MJD
## mode=1 : yyyy/mm/dd-hh:mm:ss 
## mode=2 : yyyy/doy-hh:mm:ss 
sub Epoch($$){
    my $this = shift;
    my $mode = shift;
    my $tm = $this->{epoch};
    if($mode == 0){
	return $tm->Mjd();
    }elsif($mode == 1){
	return $tm->Str_fmt3(1);
    }else{
	return $tm->Str_fmt1(1);
    }
}

sub XYZ($){
    my $this = shift;
    my $xyz = $this->{xyz};
    return @{$xyz};
}

sub XYZ_err($){
    my $this = shift;
    my $xyz_err = $this->{xyz_err};
    return @{$xyz_err};
}

sub XYZ_adj($){
    my $this = shift;
    my $xyz_adj = $this->{xyz_adj};
    return @{$xyz_adj};
}

sub Baseline($){
    my $this = shift;
    my $base = $this->{baseline};
    return @{$base};
}

sub Read($$){
    my $this =shift;
    my $fname =shift;
    my $in;
    unless(open($in,$fname)){
	print STDERR "Cannot opne file $fname \n";
	return -1;
    }
    $this->Get_stn($in);
    $this->Get_base($in);
    $this->Read_Resid($in);
    $this->Read_Clk($in);
    close($in);
    return 0;
}

sub RsdTarr($){
    my $pkg = shift;
    return @{$pkg->{rsdtarr}};
}

sub Srcarr($){
    my $pkg = shift;
    return @{$pkg->{srcarr}};
}

sub Residual($){
    my $pkg = shift;
    return   @{$pkg->{rsdarr}};
}

sub Flag($){
    my $pkg = shift;
    return   @{$pkg->{flag}};
}

sub iScan($){
    my $pkg = shift;
    return   @{$pkg->{iscan}};
}

sub ClkEpoch($){
    my $pkg = shift;
    return   @{$pkg->{clkepoch}};
}

sub ClkArr($){
    my $pkg = shift;
    return   @{$pkg->{clkarr}};
}

sub ClkErr($){
    my $pkg = shift;
    return   @{$pkg->{clkerr}};
}

sub Clk1Epoch($){ # Epoch of clock 1st order
    my $pkg = shift;
    return   $pkg->{cl1epoch};
}

sub Clk1($){ # clock 1st order
    my $pkg = shift;
    return   $pkg->{cl1};
}

sub Clk1Err($){ # clock 1st order error
    my $pkg = shift;
    return   $pkg->{cl1err};
}

sub Clk2Epoch($){ # Epoch of clock 2nd order
    my $pkg = shift;
    return   $pkg->{cl2epoch};
}

sub Clk2($){ # clock 2nd order
    my $pkg = shift;
    return   $pkg->{cl2}; ## (sec/day)
}

sub Clk2Err($){ # clock 2nd order error
    my $pkg = shift;
    return   $pkg->{cl2err};
}

sub GetClock($$){ # Compute Clock value with all clock info
    my $pkg = shift;
    my $xmjd = shift;
    my @clkarr = @{$pkg->{clkarr}};
    my @clkerr = @{$pkg->{clkerr}};
    my @clkepoch = @{$pkg->{clkepoch}};
    my $cl1 = $pkg->{cl1};
    my $cl1err = $pkg->{cl1err};
    my $cl1epoch = $pkg->{cl1epoch};
    my $cl2 = $pkg->{cl2};
    my $cl2err = $pkg->{cl2err};
    my $cl2epoch = $pkg->{cl2epoch};

    my @irange = (0,0);
    my $i=0;
    for($i=0; $i<@clkarr; $i++){
	if($clkepoch[$i]->Mjd() > $xmjd){
	    $irange[0] = $i-1;
	    $irange[1] = $i;
	    last;
	}
    }
    my $ofst;
    if($i >= @clkarr){
	    $irange[0] = $#clkarr - 1;
	    $irange[1] = $#clkarr;
    }
    $ofst = ($xmjd - $clkepoch[$irange[0]]->Mjd())*86400.*
	($clkarr[$irange[1]] -$clkarr[$irange[0]])
	/($clkepoch[$irange[1]] - $clkepoch[$irange[0]] ); 
    
    
    $ofst += $clkarr[$irange[0]];

    my $err1 = $clkerr[$irange[0]];
    my $dt1 = ($xmjd - $cl1epoch->Mjd())*86400.;
    my $dt2 = $xmjd - $cl2epoch->Mjd();
    my $err2 = $cl1err * $dt1;
    my $err3 = $cl2err * $dt2 * $dt2 * 86400.;

    my $ret = $ofst + $cl1 * $dt1; 
    $ret += $cl2 * $dt2 * $dt2 * 86400.; 
    my $err = sqrt($err1**2 + $err2**2 + $err3**2);
    return ($ret, $err);
    
}
1;
