##
package Spl_mdl;
use Exporter;
use Carp;

@ISA = qw(Exporter);
# @EXPORT = qw(Read StnXYZ Stnames);

use Time_lib;

BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {
# 	dbname => undef,
# 	UT1_TAI => undef,
# 	epoch => undef,
# 	dUT1err => undef,
    };
    bless $this;
    $this;
}

sub Read($$){
    my $pkg = shift;
    my $fname = shift;

    unless(open(IN, $fname)){
	print STDERR "Error File open failed $fname\n";
	return -1;
    }
    
    my $nitm=0;
    
    my $mode = 0;  # 0: initial, 1: Station Pos.
    my (%stxyz, %stuen);

    while(!eof(IN)){
	my $line = <IN>;
	if($line =~/Data base\s+/){
	    my $dbname = $'; #';
	    chomp $dbname;
	    $dbname = join("_", split(/\s+/,$dbname));
	    $pkg->{dbname} = $dbname;
	    $nitm++;
	    next;
	}elsif($line =~/\d+\. UT1-TAI/){
	    my @terms = split(/\s+/,$'); #')
	    my @ymd = split(m!/!,$terms[2]);
	    $ymd[0] += $ymd[0] <= 99 ? 1900 : 2000;
	    my @hm = split(m!:!,$terms[3]);
	    my $tm = new Time_lib;
	    $tm->Set_time_ymd(@ymd,@hm,0);
	    $pkg->{UT1_Epoch} = $tm;
	    $pkg->{UT1_TAI} = $terms[4];
	    $pkg->{dUT1err} = $terms[8];
	    $nitm++;
	    next;
	}elsif($line =~/hi-freq variations/){
	    for(my $i=0; $i<3; $i++){   
		$line = <IN>;}
	    my @terms = split(/\s+/,$line);
	    $pkg->{hfUT1_TAI} = $terms[4];
	    $pkg->{hfdUT1err} = $terms[7];
	    $nitm++;
	    next;
	}elsif($line =~/Residuals from/){
	    $line = <IN>; $line = <IN>; $line = <IN>;
	    my (@tarr, @rsdarr, @srcarr );
	    my $cnt=0;
	    while($line =~/(\d)\s+!\s+/){
		my @tms = split(/\s+/,$'); # ');
		if(@tms < 9){
		    print STDERR "Err: Unexpected data: $line.\n";
		    next;
		}
		my @hms = split(/[.-:]/,$tms[8]);
		my $tm = new Time_lib;
		$tm->Set_time_ymd(@hms);
		$tarr[$cnt] = $tm;
		$rsdarr[$cnt] = $tms[4] * 1.e-9;
		$srcarr[$cnt] = $tms[2];
		$cnt++;
	    }

	    $pkg->{rsdarr} = \@rsdarr;
	    $pkg->{srcarr} = \@srcarr;
	    $pkg->{tarr} = \@tarr;
	    $nitm++;
	    next;
	}elsif($line =~/Flyby Station Cals/){ # Stations 
	    my @stnames;
	    $line = <IN>;    $line = <IN>; 
	    my $cnt=0;
	    while($line !~/-------------/){
		unless($line =~/(\S+)\s*:[\s\t]*\w+/){
		    print STDERR "Warning: Station Name MisPtnmatch:[$line]\n";
		}else{
		    push(@stnames,$1);
		}
		$line = <IN>; 
	    }
	    $pkg->{stnames} = \@stnames;
	    $nitm++;
	    next;
	}elsif($line =~/Station positions.+epoch:\s*(\S+)/){
	    $mode =1; # Station Pos. mode
	    my $pos_epoch = $1;
	    my @ymdhms = split(/[:,\-,.]+/,$pos_epoch);
	    my $tm = new Time_lib;
	    $tm->Set_time_ymd(@ymdhms);
	    my @stnames = @{$pkg->{stnames}};
	    my ($st0);
	    my (@xyz, @errxyz) = ();
	    while(<IN>){
		/^\s*/;
		$line=$'; #';
		next if($line =~/^[\s\t]+$/);
		my @terms = split(/\s+/,$line);		    
		next if (@terms < 12);
		if($line =~/^\d+\.\s+([A-Z]+\w+)\s+\d+/){
		    my $st = $1;
		    unless(grep($_ eq $st,@stnames)){
			my $stl = join(",",@stnames);
			print STDERR "E: Station[$st] is not included in stations($stl)\n";
			return -1;
		    }
		    if($terms[4] eq "X"){
			$xyz[0] = $terms[6];
			$errxyz[0] = $terms[12];
			$st0 = $st;
			next;
		    }elsif($terms[4] eq "Y"){
			$xyz[1] = $terms[6];
			$errxyz[1] = $terms[12];
			unless($st0 eq $st){
			    print STDERR "E: X->Statin[$st0], but Y->$st\n";
			    next;
			};
		    }elsif($terms[4] eq "Z"){
			$xyz[2] = $terms[6];
			$errxyz[2] = $terms[12];
			unless($st0 eq $st){
			    print STDERR "E: X->Statin[$st0], but Z->$st\n";
			    next;
			};
			my $sxyz = join(" ", @xyz);
			my $exyz = join(" ", @errxyz);
			$stxyz{$st0} = "$sxyz $exyz";
		    }
		    next;
		}elsif( $line=~/^([A-Z]+\w+)\s+\d+/ ){
		    my $st = $1;
		    unless(grep($_ eq $st,@stnames)){
			my $stl = join(",",@stnames);
			print STDERR "E: Station[$st] is not included in stations($stl)\n";
			return -1;
		    }
		    if($terms[2] eq "U"){
			$xyz[0] = $terms[4];
			$errxyz[0] = $terms[10];
			$st0 = $st;
			next;
		    }elsif($terms[2] eq "E"){
			$xyz[1] = $terms[4];
			$errxyz[1] = $terms[10];
			unless($st0 eq $st){
			    print STDERR "E: U->Statin[$st0], but E->$st\n";
			    next;
			};
		    }elsif($terms[2] eq "N"){
			$xyz[2] = $terms[4];
			$errxyz[2] = $terms[10];
			unless($st0 eq $st){
			    print STDERR "E: U->Statin[$st0], but N->$st\n";
			    next;
			};
			my $sxyz = join(" ", @xyz);
			my $exyz = join(" ", @errxyz);
			$stuen{$st0} = "$sxyz $exyz";
		    }
		    next;
		}elsif($line=~/${st0}/ ){
		    next;
		}else{
		    last;
		}
	    }
	    $pkg->{StnXYZ} = \%stxyz;
	    $pkg->{StnUEN} = \%stuen;
	    $pkg->{pos_Epoch} = $tm;
	}# End of Station pos
	
    }
    close(IN);


    if($nitm <1){ return -1;}
    else { return 1; }
    
}

sub DBName($){
    my $pkg = shift;
    $pkg->{dbname};
}
## mode=1;# yyyy/mm/dd-hh:mm:ss
## otherwise || mode=0;# yyyy mm dd hh mm ss
sub UT1Epoch($;$){
    my $pkg = shift;
    my $type = shift;
    my $tm1  = $pkg->{UT1_Epoch};
    if(!defined($type) || $type == 1){
	$tm1->Str_fmt3(1);
    }else{
	$tm1->Str_fmt3(0);
    }
}
## mode=1;# yyyy/mm/dd-hh:mm:ss
## otherwise || mode=0;# yyyy mm dd hh mm ss
sub PosEpoch($;$){
    my $pkg = shift;
    my $type = shift;
    my $tm1  = $pkg->{pos_Epoch};
    if(!defined($type) || $type == 1){
	$tm1->Str_fmt3(1);
    }else{
	$tm1->Str_fmt3(0);
    }
}

sub UT1_TAI($){
    my $pkg = shift;
    $pkg->{UT1-TAI};
}

sub UT1_UTC($){
    my $pkg = shift;
    my $dut = $pkg->{UT1-TAI};
    $dut += ${$pkg->{Epoch}}->TAI_UTC();
    return $dut;
}

sub dUT1_UTCerr($){
    my $pkg = shift;
    $pkg->{dUT1err};
}

sub hfUT1_UTC($){
    my $pkg = shift;
    my $dut = $pkg->{hfUT1-TAI};
    $dut += ${$pkg->{Epoch}}->TAI_UTC();
    return $dut;
}


sub Tarr($){
    my $pkg = shift;
    @{$pkg->{tarr}};
}

sub Srcarr($){
    my $pkg = shift;
    @{$pkg->{srcarr}};
}

sub Residual($){
    my $pkg = shift;
    @{$pkg->{rsdarr}};
}

sub Stnames($){
    my $pkg = shift;
    @{$pkg->{stnames}};
}

sub StnXYZ($){
    my $pkg = shift;
    %{$pkg->{StnXYZ}};
}

sub StnUEN($){
    my $pkg = shift;
    %{$pkg->{StnUEN}};
}

1;
