package Time_lib;
use Exporter;
use Carp;
use overload
    "!=" => "notequal",
    "==" => "equal",
    ">"  => "gt",
    "<"  => "lt",
    ">="  => "ge",
    "<="  => "le",
    "-"  => "minus",
    "+"  => "plus",
    "m=" => "minuseq",
    "-=" => "minuseq",
    "p=" => "pluseq",
    "+=" => "pluseq";


use Time::Local;

@ISA = qw(Exporter);

#@EXPORT = qw(Mjd Doy Soy Sod Print_fmt1 Print_fmt2 Print_fmt3 
#	     Print_fmt4 Print_fmt10 Str_fmt3 Set_mjd2Time 
#	     Set_time_strydhms Set_time_strymdhms Set_time_ydhms, 
#            Set_time_Max);
#  Max time = 3000/1/1 

use Mjd;

BEGIN{
    @NAdoy = qw( 31 59 90 120 151 181 212 243 273 304 334 365); 
    @Month = qw(Jan Feb Mar Apr May Jun 
		Jul Aug Sep Oct Nov Dec);
    %Month = ('Jan' =>1, 'Feb' =>2, 'Mar' =>3, 'Apr' =>4,
	      'May' =>5, 'Jun' =>6, 'Jul' =>7, 'Aug' =>8,
	      'Sep' =>9, 'Oct' => 10, 'Nov' =>11, 'Dec' =>12);
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Set_time_now($;$){
    my ($pkg,$tshift) = @_;
    my @t = gmtime();
    $t[5]+=1900;
    $t[4] +=1;
#    $t[3] +=1;
    my $t1 = new Time_lib;
    $t1->Set_time_ymd($t[5],$t[4],$t[3],$t[2],$t[1],$t[0]);
    %{$pkg} = %{$t1};
    if(defined($tshift)){
	%{$pkg} = %{$t1 + $tshift};
    }
    
}

## hh mm ss.xx
sub Set_time_hms($$$$){
    my ($this,@hms) = @_;
    if(@hms < 3){
	print STDERR 
	    "Error in Time_lib::Set_time_hms(): Input data format unexpected\n";
	return -1;
    }
    $this->Set_time_ymd(2008,1,1,@hms);
    return 1;
}

## yyyy[/:.]dd hh:mm:ss
sub Set_time_strydhms($$$){
    my $this = shift;
    my $yd = shift;
    my $hms = shift;
    my @yd = split(m![/:.]!, $yd );
    my @hms = split(m!:!, $hms );
    if( @yd < 2 || @hms < 3){
	print STDERR "Error: time format is wrong : ($yd, $hms) \n";
	return(-1);
    }
    $this->Set_time_ydhms($yd[0],$yd[1], $hms[0],$hms[1],$hms[2] );
    return 1;
}
## (yy)yy[/:.]mm[/:.]dd hh:mm:ss
sub Set_time_strymdhms($$$){
    my $this = shift;
    my $ymd = shift;
    my $hms = shift;
    my @ymd = split(m![/:.]!, $ymd );
    my @hms = split(m!:!, $hms );
    if($ymd[0] < 1000){    
	$ymd[0] += ($ymd[0] <100 ? 2000 : 1900); 
    }
    if( @ymd < 3 || @hms < 3){
	print STDERR "Error: time format is wrong : ($ymd, $hms) \n";
	return(-1);
    }
    $this->Set_time_ymd(@ymd, @hms );
    return 1;
}

sub Set_time_Max($){
    my $this = shift;
    $this->Set_time_ydhms(3000,1,0,0,0); # 3000/1 0:0;0
    return 1; 
}

sub UXtime($){
    my $pkg = shift; 
    my $ymdhms = $pkg->{ymdhms};
    my $tmon = $$ymdhms[1] -1;
    my $tyear = $$ymdhms[0] - 1900;
    my $t1 = timegm($$ymdhms[5],$$ymdhms[4],$$ymdhms[3], $$ymdhms[2],$tmon, $tyear);
    return $t1;
}

sub SecFromNow($){
    my $pkg = shift;
    my $ymdhms = $pkg->{ymdhms};
    my $tmon = $$ymdhms[1] -1;
    my $tyear = $$ymdhms[0] - 1900;
    my $t1 = timegm($$ymdhms[5],$$ymdhms[4],$$ymdhms[3], $$ymdhms[2],$tmon, $tyear);
    $tnow = time();
    my $ret = $t1 - $tnow;
    return $ret;
}

sub Check_timedat($){
    my $this = shift;
    croak "Time_lib::Check_timedata(): argument must be Time_dat\n"
	unless defined($this->{ydhms});
}

sub equal($$){
    my ($this, $t1) = @_;
    Check_timedat($this);
    Check_timedat($t1);
    return ($this->Mjd() == $t1->Mjd());
}

# sub eqset($$){
#     my ($this, $t1) = @_;
#     Check_timedat($this);
#     Check_timedat($t1);
#     my @ydhms = $t1->YDhms();
#     $this->Set_time_ydhms(@ydhms);
# }


sub notequal($$){
    my ($this, $t1) = @_;
    Check_timedat($this);
    Check_timedat($t1);
    return ($this->Mjd() != $t1->Mjd());
}

sub gt($$){
    my ($this, $t1) = @_;
    Check_timedat($t1);
    Check_timedat($this);
    return ($this->Mjd()  > $t1->Mjd());
}

sub lt($$){
    my ($this, $t1) = @_;
    Check_timedat($this);
    Check_timedat($t1);
    return ($this->Mjd() < $t1->Mjd());
}

sub ge($$){
    my ($this, $t1) = @_;
    Check_timedat($t1);
    Check_timedat($this);
    return ($this->Mjd()  >= $t1->Mjd());
}

sub le($$){
    my ($this, $t1) = @_;
    Check_timedat($this);
    Check_timedat($t1);
    return ($this->Mjd() <= $t1->Mjd());
}

sub minus($$){
    my ($this, $t1) = @_;
    Check_timedat($this);
    Check_timedat($t1);
    return ($this->Mjd() - $t1->Mjd())*86400.;
}

sub plus($$){
    my ($this, $sec) = @_;
    Check_timedat($this);
    my $mjd = $this->Mjd();
    $mjd += $sec/86400.;
    my $t1 = new Time_lib;
    $t1->Set_mjd2Time($mjd);
    
    return $t1;
}

sub pluseq($$){
    my ($this, $sec) = @_;
    Check_timedat($this);
    my $mjd = $this->Mjd();
    $mjd += $sec/86400.;
    $this->Set_mjd2Time($mjd);
    return $this;
}

sub minuseq($$){
    my ($this, $sec) = @_;
    Check_timedat($this);
    my $mjd = $this->Mjd();
    $mjd -= $sec/86400.;
    $this->Set_mjd2Time($mjd);
    return $this;
}

sub YDHms($){
    my $this =shift;
    my $ydhms = $this->{ydhms};
    return ($$ydhms[0],$$ydhms[1],$$ydhms[2], $$ydhms[3],$$ydhms[4]);
}

sub Hms($){
    my $this =shift;
    my $ydhms = $this->{ydhms};
    return ($$ydhms[2], $$ydhms[3],$$ydhms[4]);
}


## Example:  "2004 3 14 00 00 00"
sub Set_time_ymd($$$$$$$){
    my $this =shift;
    my @ymdhms;
    for($i=0; $i<6 && @_ > 0; $i++){
	$ymdhms[$i]= shift;
    }
    if(@ymdhms < 3){
	print STDERR "Time_lib object Initialization Error: time data is @ymdhms \n";
	return($this);
    }
    $this->{ymdhms} = \@ymdhms;
    my $doy =  $this->{doy} = &doy($ymdhms[0], $ymdhms[1], $ymdhms[2]);
    my $sod=0;
    $sod += $ymdhms[3]*3600. if(defined($ymdhms[3]));
    $sod += $ymdhms[4]*60. if(defined($ymdhms[4]));
    $sod += $ymdhms[5] if(defined($ymdhms[5]));
    $this->{sod} = $sod;
    $this->{soy} = ($doy-1)*86400.+ $sod;
    $this->{mjd} = &mjd($ymdhms[0], $ymdhms[1], $ymdhms[2]) + $sod/86400.;
    my @ydhms = ($ymdhms[0], $doy, $ymdhms[3],$ymdhms[4],$ymdhms[5]);
    $this->{ydhms} = \@ydhms;
    $this;
}

sub Canonic_ydhms($){
    my $this =shift;
    my @ydhms = @$this;
    if(@ydhms < 5){
	my $ret = join("_",@ydhms);
	print STDERR "Err at Canonic_ydhms(): array '$ret' does not contain 5 terms\n";
	exit(1);
    }
    my $cary = $ydhms[4]/60.;
    if( $cary>= 1.){
	$ydhms[4] = $ydhms[4] % 60.;
	$ydhms[3] += $cary;
    }
    $cary = $ydhms[3]/60.;
    if( $cary>= 1.){
	$ydhms[3] = $ydhms[3] % 60.;
	$ydhms[2] += $cary;
    }
    $cary = $ydhms[2]/24.;
    if( $cary>= 1.){
	$ydhms[2] = $ydhms[2] % 24.;
	$ydhms[1] += $cary;
    }
    my $leap = 0;
    my $yy = $ydhms[0];
    $leap = 1 if( (($yy % 4 == 0) && ($yy % 100 != 0)) || ($yy % 400 == 0));
    my $maxday = 365 + $leap;
    if($ydhms[1] > $maxday){
	$ydhms[1] -= $maxday;
	$ydhms[0]++;
    }

    return @ydhms;
}

 ## Example yyyy ddd hh mm ss 
sub Set_time_ydhms($$$$$$){
    my $this =shift;
    my (@ydhms1, @ymdhms);
    for($i=0; $i<5 && @_ > 0; $i++){
	$ydhms1[$i]= shift;
    }
    if(@ydhms1 < 3){
	print STDERR "Time_lib object Initialization Error: time data is @ydhms1 \n";
	return($this);
    }
    my @ydhms = Canonic_ydhms(\@ydhms1);
    $this->{ydhms} = \@ydhms;
    my $doy =  $this->{doy} = $ydhms[1];
    my $sod=0;
    $sod += $ydhms[2]*3600. if(defined($ydhms[2]));
    $sod += $ydhms[3]*60. if(defined($ydhms[3]));
    $sod += $ydhms[4] if(defined($ydhms[4]));
    $this->{sod} = $sod;
    $this->{soy} = ($doy-1)*86400.+ $sod;
    $this->{mjd} = &mjd($ydhms[0], 1, 1) -1 
	+ $doy + $sod/86400.;
    $ymdhms[0] = $ydhms[0];
    $ymdhms[1] = $ymdhms[1] = 0;
    $ymdhms[3] = $ydhms[2];
    $ymdhms[4] = $ydhms[3];
    $ymdhms[5] = $ydhms[4];
    $this->{ymdhms} = \@ymdhms;
    $this->Doy2md;
    $this;
}

## Example:  "2004 Sep 28 12 21 00" 
sub Set_time_dMyhms($$$$$$$){
    my $this =shift;
    my @ymdhms;
    for($i=0; $i<6 && @_ > 0; $i++){
	my $tmp= shift;
	my $tmp2 = $Month{$tmp} if( $i ==1 );
	$ymdhms[$i]= $tmp2;
    }
    if(@ymdhms < 3){
	print STDERR "Time_lib object Initialization Error: time data is @ymdhms \n";
	return($this);
    }

    $this->{ymdhms} = \@ymdhms;
    my $doy =  $this->{doy} = &doy($ymdhms[0], $ymdhms[1], $ymdhms[2]);
    my $sod=0;
    $sod += $ymdhms[3]*3600. if(defined($ymdhms[3]));
    $sod += $ymdhms[4]*60. if(defined($ymdhms[4]));
    $sod += $ymdhms[5] if(defined($ymdhms[5]));
    $this->{sod} = $sod;
    $this->{soy} = ($doy-1)*86400.+ $sod;
    $this->{mjd} = &mjd($ymdhms[0], $ymdhms[1], $ymdhms[2]) + $sod/86400.;
    my @ydhms = ($ymdhms[0], $doy, $ymdhms[3],$ymdhms[4],$ymdhms[5]);
    $this->{ydhms} = \@ydhms;
    $this;
}
sub Set_time_Mjd($$){
    my $pkg = shift;
    my $mjd = shift;
    $pkg->Set_mjd2Time($mjd);
}

sub Set_mjd2Time($$){
    my ($this,$mjd_in) = @_;
    my $mjd_2000 = &mjd(2000,1,1);

    my @ydhms;
    if($mjd_in > $mjd_2000){
	my $mjd_tmp = $mjd_2000;
	my $yy = 2000;
	while($mjd_in > $mjd_tmp){
	    $yy++;
	    $mjd_tmp = &mjd($yy,1,1);
	}
	$ydhms[0] = $yy-1;
    }else{
	my $mjd_tmp = $mjd_2000;
	my $yy = 2000;
	while($mjd_in < $mjd_tmp){
	    $yy--;
	    $mjd_tmp = &mjd($yy,1,1);
	}
	$ydhms[0] = $yy;
    }

    my $doy = $mjd_in - &mjd($ydhms[0],1,1)+1. + 1.e-11;
    $this->{doy} = int($doy);
    $doy -= $this->{doy};
    my $sod = $doy * 86400;
    $this->{sod} = $sod;
    my $hh = int($sod / 3600);
    $sod -= $hh * 3600.;
    my $mm = int($sod / 60 );
    $sod -= $mm * 60.;
    $ydhms[1] = $this->{doy};
    $ydhms[2] = $hh;
    $ydhms[3] = $mm;
    $ydhms[4] = int($sod*1000)/1000.;
    $this->{ydhms} = \@ydhms;
    $this->Doy2md;
    $this->{mjd} = $mjd_in;
    $this->{soy} = ($this->{doy}-1) * 86400. + $this->{sod};
    $this;
};

sub Doy2md($)
{
    my $this = shift;
    my $doy = $this->{doy};
    my $ydhms = $this->{ydhms};
    my $ymdhms = $this->{ymdhms};
    my $leap=0;
    my $yy = defined($$ymdhms[0]) ? $$ymdhms[0] : $$ydhms[0];
    $$ymdhms[0] = $yy;
    $leap =1 if(( ($yy % 4) == 0 && ($yy % 100 ) !=0) || ($yy % 400) == 0);
    my @ndoy = @NAdoy;
    for($i=1; $i<12; $i++){ $ndoy[$i] += $leap;   }
    
    for($i=0; $i<12; $i++){last if($doy <= $ndoy[$i] ); }
    $$ymdhms[1] = $i+1;
    if($i==0){
	$$ymdhms[2] = $doy;
    }else{
	$$ymdhms[2] = $doy - $ndoy[$i-1]; 
    }
    $$ymdhms[3] = $$ydhms[2];
    $$ymdhms[4] = $$ydhms[3];
    $$ymdhms[5] = $$ydhms[4];
    $this->{ymdhms} = $ymdhms;
    $this->{ydhms} = $ydhms;
}

sub Year($){
    my $this = shift;
    ${$this->{ydhms}}[0];
}

sub Doy($){
    my $this = shift;
    $this->{doy};
}

sub Soy($){
    my $this = shift;
    $this->{soy};
}

sub Sod($){
    my $this = shift;
    $this->{sod};
}
sub Mjd($){
    my $this = shift;
    $this->{mjd};
}

sub Print_fmt1($;$){ # yyyy/ddd hh:mm:ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	if($out !~/^main/){ $H = $p . "::" . $out;}
	else{ $H =  $out;}
    }else{
	$H = STDOUT;
    }
    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	printf $H "%04d/%03d %02d:%02d:%06.3f ",
	$$ymdhms[0],$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	printf $H "%04d/%03d %02d:%02d:%06.3f ",
	$$ydhms[0],$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
    
}

sub Print_fmt10($;$){ # yyyy ddd hh:mm:ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	$H = $p . "::" . $out;
    }else{
	$H = STDOUT;
    }
    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	printf $H "%04d %03d %02d:%02d:%06.3f ",
	$$ymdhms[0],$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	printf $H "%04d %03d %02d:%02d:%06.3f ",
	$$ydhms[0],$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
    
}
sub Print_fmt13($;$){ # yyyy.ddd.hh:mm:ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	$H = $p . "::" . $out;
    }else{
	$H = STDOUT;
    }
    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	printf $H "%04d.%03d.%02d:%02d:%06.3f ",
	$$ymdhms[0],$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	printf $H "%04d.%03d.%02d:%02d:%06.3f ",
	$$ydhms[0],$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
}


sub Print_fmt2($$){ # yyyy/mm/dd hh:mm:ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H = $p . "::" . $out;
    my @ymdhms = @{$this->{ymdhms}};
    printf $H "%04d/%02d/%02d %02d:%02d:%06.3f %12.6f",
    $ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5], $this->{mjd};
}
sub Print_fmt21($$){ # yyyy/mm/dd-hh:mm:ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H = $p . "::" . $out;
    my @ymdhms = @{$this->{ymdhms}};
    printf $H "%04d/%02d/%02d-%02d:%02d:%06.3f %12.6f",
    $ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5], $this->{mjd};
}

sub Print_fmt3($$){ # yyyy mm dd hh mm ss
    my ($this, $out) = @_;
    my $p = caller;
    my $H = $p . "::" . $out;
    my @ymdhms = @{$this->{ymdhms}};
    printf $H "%04d %02d %02d %02d %02d %02d",
    $ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5];
}
sub Print_fmt4($$){ # (yyyy)y(ddd)d(hh)h(mm)m(ss)s
    my ($this, $out) = @_;
    my $p = caller;
    my $H = $p . "::" . $out;
    my @ydhms = @{$this->{ydhms}};
    printf $H "%04dy%03dd%02dh%02dm%02ds",
    $ydhms[0], $ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4];
}

sub Str_fmt_src1($){ # hhmmss.xx
    my ($this) = shift;
    my @ymdhms = @{$this->{ymdhms}};
    sprintf "%02d%02d%05.2f",
    $ymdhms[3],$ymdhms[4],$ymdhms[5];
}

## mode=0;# yyyy mm dd hh mm ss
## mode=1;# yyyy/mm/dd-hh:mm:ss
## mode=2;# yyyy/mm/dd hh:mm:ss
sub Str_fmt3($;$){ # yyyy mm dd hh mm ss
    my ($this) = shift;
    my $mode = shift;
    my @ymdhms = @{$this->{ymdhms}};
    my @ydhms = @{$this->{ydhms}};
    if(!defined($mode) || $mode ==0){
	sprintf "%04d %02d %02d %02d %02d %02d",
	$ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5];
    }elsif($mode ==1){
	sprintf "%04d/%02d/%02d-%02d:%02d:%02d",
	$ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5];
    }elsif($mode ==2){
	sprintf "%04d/%02d/%02d %02d:%02d:%02d",
	$ymdhms[0], $ymdhms[1],$ymdhms[2],$ymdhms[3],$ymdhms[4],$ymdhms[5];
    }else{
	print STDERR "E Time_lib::Str_fmt3(): mode=$mode is not defined\n ";
	exit(1);
    }
}

sub Str_fmt12($;$){ # yydddhhmmss
    my ($this) = @_;

    my $str;
    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	my $yy = $$ymdhms[0] % 100;
	$str = sprintf "%02d%03d%02d%02d%02d",
	$yy ,$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	my $yy = $$ymdhms[0] % 100;
	$str = sprintf "%02d%03d%02d%02d%02d",
	$yy,$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
    $str;
}

sub Str_fmt11($;$){ # yyyydddhhmmss
    my ($this) = @_;

    my $str;
    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	$str = sprintf "%04d%03d%02d%02d%02d",
	$$ymdhms[0],$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	$str = sprintf "%04d%03d%02d%02d%02d",
	$$ydhms[0],$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
    $str;
}

## mode ==0 # yyyy/ddd hh:mm:ss
## mode ==1 # yyyy/ddd-hh:mm:ss
## mode ==2 # yyyy.ddd.hh:mm:ss
## mode ==21 # yyyy.ddd.hh:mm:ss.xx
sub Str_fmt1($;$){ # yyyy/ddd hh:mm:ss
    my ($this,$mode) = @_;

    if(defined($mode)){
	if($mode ==1){
	    my $ymdhms = $this->{ymdhms};
	    my $str = sprintf "%04d/%03d-%02d:%02d:%02d",
	    $$ymdhms[0], $this->{doy}, 
	    $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
	    return $str;
	}elsif($mode ==2){
	    my $ymdhms = $this->{ymdhms};
	    my $str = sprintf "%04d.%03d.%02d:%02d:%02d",
	    $$ymdhms[0], $this->{doy}, 
	    $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
	    return $str;
	}elsif($mode ==21){
	    my $ymdhms = $this->{ymdhms};
	    my $str = sprintf "%04d.%03d.%02d:%02d:%05.2f",
	    $$ymdhms[0], $this->{doy}, 
	    $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
	    return $str;
	}
    }

    if (defined($this->{ymdhms})){
	my $ymdhms = $this->{ymdhms};
	my $doy = $this->{doy};
	my $str = sprintf "%04d/%03d %02d:%02d:%02d",
	$$ymdhms[0],$doy, $$ymdhms[3],$$ymdhms[4],$$ymdhms[5];
	return $str;
    }elsif(defined($this->{ydhms})){
	my $ydhms = $this->{ydhms} ;
	my $str = sprintf "%04d/%03d %02d:%02d:%02d",
	$$ydhms[0],$$ydhms[1],$$ydhms[2],$$ydhms[3],$$ydhms[4];
	return $str;
    }else{
	print STDERR "Error: time_data is not defined\n";
	return(-1);
    }
}

sub ord_time_str($$)
{
    my ($pkg,$fmt) = @_;
    my @dum = split(/%/,$fmt);
    my $num = @dum -1;
    my @nim;
    if( $num < 5 || $num >6){
	print "String Format is wrong: $fmt \n";
	exit(1);
    }
    
    my (%has, $i);
    if( $num == 6){
	if ($fmt =~ /%Y/){ 
	    @nim = ( '%Y', '%Mon', '%Day', '%H', '%M', '%S');
	}else{
	    @nim = ( '%y', '%Mon', '%Day', '%H', '%M', '%S');
	}
	for( $i=0; $i< 6; $i++){
	    $has{$i} = index($fmt, $nim[$i], 0);
	    if( $has{$i} <0){
		print "Error: $nim[$i] was not found in the string format($fmt)\n";
		exit(1);
	    }
	    if( $nim[$i] =~m!%Y!){
		$fmt =~s/$nim[$i]/\\s*(\\d\{4\})/;
	    }elsif($nim[$i] =~m!%y!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{2\})!;
	    }elsif($nim[$i] =~m!%Mon!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\})!;
	    }elsif($nim[$i] =~m!%Day!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\})!;
	    }elsif($nim[$i] =~m!%S!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\}\\.\*\\d\*)!;
	    }else{
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\})!;
	    }
	}
    }else{
	if ($fmt =~ /%Y/){ 
	    @nim = ( '%Y', '%Doy', '%H', '%M', '%S');
	}else{
	    @nim = ( '%y', '%Doy', '%H', '%M', '%S');
	}
	for( $i=0; $i< 5; $i++){
	    $ofst = $has{$i} = index($fmt, $nim[$i], 0);
	    if( $has{$i} <0){
		print "Error: $nim[$i] was not found in the string format($fmt)\n";
		exit(1);
	    }
	    if( $nim[$i] =~m!%Y!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{4\})!;
	    }elsif($nim[$i] =~m!%y!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{2\})!;
	    }elsif($nim[$i] =~m!%Doy!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{3\})!;
	    }elsif($nim[$i] =~m!%S!){
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\}\\.\*\\d\*)!;
	    }else{
		$fmt =~s!$nim[$i]!\\s*(\\d\{1,2\})!;
	    }
	}
    }
    my  @ord = sort {$has{$a} <=> $has{$b}} keys %has;
#    print "ord = ",@ord,  " fmt = $fmt \n";
    $pkg->{ord} = \@ord;
    $pkg->{fmt} = $fmt;
    return $num;
}

sub Tfmt($){
    my $pkg = shift;
    $pkg->{fmt};
}

sub Tord($){
    my $pkg = shift;
    $pkg->{ord};
}

sub T_Match($$){
    my ($pkg,$line) = @_;
    my $ord = $pkg->{ord};
    my $fmt = $pkg->{fmt};
    my $nitm = @{$ord};
    my @tmparr;
    
    unless($line=~/$fmt/){
	print STDERR "pattern matching failed, Format:$fmt != $line \n";
	return -1;
    }

    if( $nitm == 5){
	$tmparr[$$ord[0]] = $1;
	$tmparr[$$ord[1]] = $2;
	$tmparr[$$ord[2]] = $3;
	$tmparr[$$ord[3]] = $4;
	$tmparr[$$ord[4]] = $5;
	if( $tmparr[0] < 100){
	    $tmparr[0] += $tmparr[0] > 50 ? 1900 : 2000;
	}
	$pkg->Set_time_ydhms(@tmparr);
    }elsif( $nitm == 6){
	$tmparr[$$ord[0]] = $1;
	$tmparr[$$ord[1]] = $2;
	$tmparr[$$ord[2]] = $3;
	$tmparr[$$ord[3]] = $4;
	$tmparr[$$ord[4]] = $5;
	$tmparr[$$ord[5]] = $6;
	if( $tmparr[0] < 100){
	    $tmparr[0] += $tmparr[0] < 50 ? 2000 : 1900;
	}
	$pkg->Set_time_ymd(@tmparr);
    }
    return 1;
}

sub TAI_UTC($){
    my $pkg = shift;
    
    my $n=37;
    my @tai_utc_cff = 
	(
	 [ 2437300.5, 1.4228180, 37300., 0.001296],
	 [ 2437512.5, 1.3728180, 37300., 0.001296],
	 [ 2437665.5, 1.8458580, 37665., 0.0011232],
	 [ 2438334.5, 1.9458580, 37665., 0.0011232],
	 [ 2438395.5, 3.2401300, 38761., 0.001296],
	 [ 2438486.5, 3.3401300, 38761., 0.001296],
	 [ 2438639.5, 3.4401300, 38761., 0.001296],
	 [ 2438761.5, 3.5401300, 38761., 0.001296],
	 [ 2438820.5, 3.6401300, 38761., 0.001296],
	 [ 2438942.5, 3.7401300, 38761., 0.001296],
	 [ 2439004.5, 3.8401300, 38761., 0.001296],
	 [ 2439126.5, 4.3131700, 39126., 0.002592],
	 [ 2439887.5, 4.2131700, 39126., 0.002592],
	 [ 2441317.5, 10.0, 41317., 0.0],
	 [ 2441499.5, 11.0, 41317., 0.0],
	 [ 2441683.5, 12.0, 41317., 0.0],
	 [ 2442048.5, 13.0, 41317., 0.0],
	 [ 2442413.5, 14.0, 41317., 0.0],
	 [ 2442778.5, 15.0, 41317., 0.0],
	 [ 2443144.5, 16.0, 41317., 0.0],
	 [ 2443509.5, 17.0, 41317., 0.0],
	 [ 2443874.5, 18.0, 41317., 0.0],
	 [ 2444239.5, 19.0, 41317., 0.0],
	 [ 2444786.5, 20.0, 41317., 0.0],
	 [ 2445151.5, 21.0, 41317., 0.0],
	 [ 2445516.5, 22.0, 41317., 0.0],
	 [ 2446247.5, 23.0, 41317., 0.0],
	 [ 2447161.5, 24.0, 41317., 0.0],
	 [ 2447892.5, 25.0, 41317., 0.0],
	 [ 2448257.5, 26.0, 41317., 0.0],
	 [ 2448804.5, 27.0, 41317., 0.0],
	 [ 2449169.5, 28.0, 41317., 0.0],
	 [ 2449534.5, 29.0, 41317., 0.0],
	 [ 2450083.5, 30.0, 41317., 0.0],
	 [ 2450630.5, 31.0, 41317., 0.0],
	 [ 2451179.5, 32.0, 41317., 0.0],
	 [ 2453736.5, 33.0, 41317., 0.0] );
    
    my $mjd = $pkg->Mjd();
    my $i;
    for($i=0; $i<$n; $i++){
	last if( $mjd < $tai_utc_cff[$i][0]-2400000.5);
    }
    my $ip = $i-1;
    my $xmjd = $tai_utc_cff[$ip][2];
    my $tai_utc = $tai_utc_cff[$ip][1] + ($mjd - $xmjd)*$tai_utc_cff[$ip][3];
    return ( $tai_utc);
}

1;
